/**
* @file grunt minification and concatenation configuration.
*
* TODO Create all tasks in code, output to grunt.json as we are doing with the scriptlinker files.
*
* NOTE!!!!! Before modifying this file be sure to back it up.
* Although grunt configs are not terribly diffcult they can be tricky
* when dealing with trailing slashes, asterisks in the wrong place etc.
*
* @see {@link http://gruntjs.com/configuring-tasks|Grunt} configuration examples.
*/
module.exports = function (grunt) {

    'use strict';

    // Path to dependencies from global Xipframe install.

    var gruntJson, src, isAngular, isLess, client, assets, environment,
        lessDest, lessSrc, lessFilename, angularMinify, angularBootstrapper, angularApp, viewEngine,
        lessSrcRoot, lessDestRoot, cleanCss, cleanJs;

    /* always enable force */
    grunt.option('force', true);

    /* load grunt.json*/
    gruntJson = grunt.file.readJSON(grunt.option('config') || 'grunt.json');

    /* set grunt source */
    src = gruntJson.src;
    viewEngine = gruntJson.viewEngine || 'ejs';
    environment = gruntJson.environment;

    /* set relative roots */
    assets = gruntJson.assets;
    client = gruntJson.client;

    /* paths to clean dirs */
    cleanCss = client + '/public/css';
    cleanJs = client + '/public/js';

    /* angular */
    isAngular = gruntJson.angular || false;
    angularMinify = gruntJson.angularMinify || false;
    angularApp = gruntJson.angularApp;
    angularBootstrapper = gruntJson.angularBootstrapper;

    /* LESS */
    isLess = gruntJson.less || false;
    lessSrc = gruntJson.lessSrc;
    lessDest = gruntJson.lessDest;
    lessSrcRoot = gruntJson.lessSrcRoot;
    lessDestRoot = gruntJson.lessDestRoot;

    /* can't find dependencies without path */
    if (!src) throw new Error('Grunt failed invalid path to dependencies.');

    /* load modules */
    grunt.loadTasks(src + '/grunt-contrib-concat/tasks'); // concats files.
    grunt.loadTasks(src + '/grunt-scriptlinker/tasks'); // links scripts to your layout or index page.
    grunt.loadTasks(src + '/grunt-contrib-watch/tasks'); // watches files for changes.
    grunt.loadTasks(src + '/grunt-contrib-uglify/tasks'); // minifies js.
    grunt.loadTasks(src + '/grunt-contrib-cssmin/tasks'); // minifies css.
    grunt.loadTasks(src + '/grunt-contrib-less/tasks'); // compiles/minifies LESS.
    grunt.loadTasks(src + '/grunt-contrib-clean/tasks'); // cleans a directory.
    grunt.loadTasks(src + '/grunt-contrib-copy/tasks'); // copies files.

    grunt.initConfig({

        /* clean output directory */
        clean: [cleanCss, cleanJs],

        /* minify JavaScript */
        uglify: {
            mixin: {
                src: assets + '/mixins/**/*.js',
                dest: client + '/public/js/mixins.js'
            },
            minify: {
                expand: true,
                cwd: assets + '/minify/js',
                src: ['**/*.js'],
                dest: client + '/public/js/',
                ext: '.js'
            },
            angular: {
                src: [(assets + '/angular/**/*.js'), ('!' + assets + ('/angular/' + angularBootstrapper))],
                dest: client + '/angular/' + angularApp
            }
        },

        /* build css */
        cssmin: {
            minify: {
                expand: true,
                cwd: assets + '/minify/css',
                src: ['**/*.css'],
                dest: client + '/public/css/',
                ext: '.css'
            },
            mixin: {
                src: assets + '/mixins/css/**/*.css',
                dest: client + '/public/css/mixins.css'
            }
        },

        /* concat files */
        concat: {
            js: {
                src: [(assets + '/mixins/js/**/*.js')],
                dest: client + '/public/js/mixins.js'
            },
            css: {
                src: [(assets + '/mixins/css/**/*.css')],
                dest: client + '/public/css/mixins.css'
            },
            angular: {
                src: [(assets + '/angular/**/*.js'), ('!' + assets + ('/angular/' + angularBootstrapper))],
                dest: client + '/angular/' + angularApp
            }
        },

        /* copy files only used in development */
        copy: {
            main: {
                files: [
				  { expand: true, cwd: assets + '/minify/css', src: ['**/*'], dest: client + '/public/css/' },
				  { expand: true, cwd: assets + '/minify/js', src: ['**/*'], dest: client + '/public/js/' }
                ]
            },
            angular: {
                files: [
                  { src: assets + '/angular/' + angularBootstrapper, dest: client + ('/angular/' + angularBootstrapper) }
                ]
            }
        },

        /* compile less to css */
        less: {
            development: {
                options: { paths: [(assets + '/less')] },
                files: [
                      { src: ['*.less'], dest: lessDestRoot, expand: true, ext: '.css', cwd: (assets + '/less/') }
                ]
            },
            production: {
                options: { paths: [(assets + '/less')], yuicompress: true },
                files: [
					 { src: ['*.less'], dest: lessDestRoot, expand: true, ext: '.css', cwd: (assets + '/less/') }
                ]
            }
        },

        scriptlinker: gruntJson.linker,

        watch: {
            assets: {
                files: ['assets/**/*'],
                tasks: environment,
                options: {
                    spawn: false
                }
            }
        }

    });


    /* register tasks */
    var devTasks = ['clean', 'concat:js', 'concat:css', 'copy:main', 'scriptlinker']
      , prodTasks = ['clean', 'uglify:mixin', 'uglify:minify', 'cssmin', 'scriptlinker']
	  , watchTasks;

    /* push both only one will get called though */
    if (isLess) {
        devTasks.push('less:development');
        prodTasks.push('less:production');
    }

    /* push both only one will get called though */
    if (isAngular) {
        if (angularMinify) {
            prodTasks.push('uglify:angular', 'copy:angular');
            devTasks.push('concat:angular', 'copy:angular');
        } else {
            prodTasks.push('concat:angular', 'copy:angular');
            devTasks.push('concat:angular', 'copy:angular');
        }
    }

    /* check if watching is enabled */
    if (gruntJson.watch && environment === 'development') {
        devTasks.push('watch');
        watchTasks = devTasks.slice(0);
        remove(watchTasks, 'scriptlinker');
        remove(watchTasks, 'watch');
    }

    function remove(arr, item) {
        for (var i = arr.length; i--;) {
            if (arr[i] === item) {
                arr.splice(i, 1);
            }
        }
    }


    grunt.registerTask('development', devTasks);

    grunt.registerTask('production', prodTasks);

};
