

/**
* Calls grunt command line via child process.
*
* TODO need to oranize this better for grunt.json output.
*
* @module grunt
* @param {object} xipframe - xipframe instance.
*/
module.exports = function grunt(xipframe) {

    'use strict';

    var config = xipframe.config
      , utils = xipframe.utils
      , helpers = utils.helpers
      , _ = utils._;

    return function runTask(task) {

        /* don't run grunt if starting from remote */
        if (!config.grunt.enabled) return false;

        /* since we are outside the root of our app grunt needs to change to its directory */
        if (xipframe.remote) process.chdir(config.paths.app);

        if (!task) throw new Error('Grunt task not specified or is unrecognized.');

        var env = config.environment, gruntPath = 'grunt ',
            exec = require('child_process').exec,
            cmd = '', assets = config.paths.assets.split('/').pop(),
            client = config.paths.client.split('/').pop(),
            pkg = {}, child,
            viewRoot = client + '/views',
            //viewIndex = viewRoot + config.linker.index,
            srcRoot = client + '/public';

        /* module src, env and view engine */
        pkg.src = config.paths.root + '/node_modules';
        pkg.environment = env;
        pkg.viewEngine = config.viewEngine;

        /* assets and client build dirs */
        pkg.assets = assets;
        pkg.client = client;

        /* options for less */
        pkg.less = config.less.enabled;
        pkg.lessSrc = assets + '/less/' + config.less.lessFilename + '.less';
        pkg.lessDest = client + '/public/css/' + config.less.lessFilename + '.css';
        pkg.lessSrcRoot = assets + '/less/';
        pkg.lessDestRoot = client + '/public/css/';

        /* options for Angular */
        pkg.angular = config.angular.enabled;
        pkg.angularMinify = config.angular.minify;
        pkg.angularApp = config.angular.app;
        pkg.angularBootstrapper = config.angular.bootstrapper || 'bootstrapper.js';

        /* whether or not to watch files */
        pkg.watch = config.grunt.watch;

        function checkPrefix(str) {
            if (!str.charAt(0) == '/')
                str = '/' + str;
            return str;
        }

        /* if script linker is enabled setup link scripts/styles */
        if (config.linker.enabled) {

            /* add linker files */
            pkg.linker = {
                scripts: {
                    options: {
                        startTag: config.linker.templates[config.viewEngine].scriptsTag,
                        endTag: config.linker.templates[config.viewEngine].scriptsEndTag,
                        fileTmpl: config.linker.templates[config.viewEngine].script,
                        appRoot: srcRoot
                    },
                    files: {}
                },
                styles: {
                    options: {
                        startTag: config.linker.templates[config.viewEngine].stylesTag,
                        endTag: config.linker.templates[config.viewEngine].stylesEndTag,
                        fileTmpl: config.linker.templates[config.viewEngine].style,
                        appRoot: srcRoot
                    },
                    files: {}
                }
            };

            /* 
            * build up linker paths/files 
            */
            var viewIndex, styleFiles, scriptFiles, linkerTpl, linkerItems, indexIsArray, linkerIndexViews
            
            styleFiles = pkg.linker.styles.files;
            scriptFiles = pkg.linker.scripts.files;
            linkerItems = '';
            indexIsArray = _.isArray(config.linker.index);      
            viewIndex = indexIsArray ? checkPrefix(config.linker.index[0]) : checkPrefix(config.linker.index);
            viewIndex = viewRoot + viewIndex;
            linkerIndexViews = indexIsArray === true ? "[" + ("'" + config.linker.index.join("', '") + "'") + "]" : "'" + config.linker.index + "'";

            _.forEach(config.linker.files, function (file, idx) {

                var tab = idx === 0 ? '' : '        '
                  , newLine = idx + 1 === config.linker.files.length ? '' : '\n'
                  , fileEnabled = true
                  , type
                  , src
                  , enabled
                  , replace
                  , view
                  , enabled;
         
                type = file.type ? "type: '" + file.type + "'" : "type: '" + "custom" + "'";
                src = file.src ? ", src: '" + file.src + "'" : "";
                enabled = file.enabled === true || file.enabled === false ? ", enabled: " + file.enabled : "";
                replace = file.replace ? ", replace: '" + file.replace + "'" : "";
                view = file.view ? ", view: '" + file.view + "'" : "";
             
                linkerItems += "" + tab + "{ " + type + src + replace + enabled + view + " }," + newLine + "";

                if (file.enabled === false) fileEnabled = false;

                if (file.src && fileEnabled) {

                    var type = file.src.slice(file.src.lastIndexOf('.') + 1, file.src.length)
                      , isUrl = (file.src.indexOf('http') !== -1) || file.src.indexOf('//') !== -1;

                    /* make sure there is a view may be using default */
                    if (file.view) file.view = viewRoot + checkPrefix(file.view);
                    else file.view = viewIndex;                  

                    /* replace if required */
                    if (file.replace && file.replace.length > 0) file.src = file.src.replace('[REPLACE]', file.replace);

                    /* link file to index page.*/
                    if (type === 'js') {                        
                        var prefix = !isUrl ? srcRoot : '';
                        if (!scriptFiles[file.view]) scriptFiles[file.view] = [];
                        scriptFiles[file.view].push(prefix + file.src);            
                    } else {
                        var prefix = isUrl ? srcRoot : '';
                        if (!styleFiles[file.view]) styleFiles[file.view] = [];
                        styleFiles[file.view].push(prefix + file.src);                                          
                    }
                }
                
            });


            ///* update the linker file and save */
            linkerTpl = utils.diskio.readSync(config.paths.templates + '/linker.tpl');
            linkerTpl = linkerTpl.replace('[INDEX]', linkerIndexViews);
            linkerTpl = linkerTpl.replace('[ENABLED]', config.linker.enabled);
            linkerTpl = linkerTpl.replace('[FILES]', linkerItems);
            utils.diskio.writeSync(config.paths.configure + '/linker.js', linkerTpl);

            /* if multiple linker.index duplicate files for that view */
            if (indexIsArray && config.linker.index.length > 1) {
                var curScripts = scriptFiles[viewIndex] || null
                  , curStyles = styleFiles[viewIndex] || null;
                _.forEach(config.linker.index, function (v, idx) {
                    v = viewRoot + checkPrefix(v);
                    if (curScripts) scriptFiles[v] = curScripts;
                    if (curStyles) styleFiles[v] = curStyles;
                });         
            }

            /* remove linker event if files object is empty */
            if (_.isEmpty(styleFiles)) delete pkg.linker.styles;
            if (_.isEmpty(scriptFiles)) delete pkg.linker.scripts;

        }
        
        /*
        * write the package to the app dir. makes grunt less dependent on args passed
        * instead grunt gets config from grunt.json so user can run grunt directly
        * and still properly build.
        */
        utils.diskio.writeSync(config.paths.app + '/grunt.json', JSON.stringify(pkg, null, 4));

        /* build grunt command */
        /* force build so that grunt ignores warnings for empty dirs during uglify. */
        cmd = gruntPath + task + ' --force' + ' --base="' + config.paths.app + '" --config="' + (config.paths.app + '/grunt.json"');

        /* create the child process and exec command */
        child = exec(cmd, {}, function (err, stdout, stderr) {
            //if (err) xipframe.log.error(err);
            //if (stderr) xipframe.log.error(stderr);
        });

        // On exit remove the process from child object.      
        child.on('exit', function() {
            utils.helpers.removeByProperty(xipframe.children, 'pid', child.pid);
            child.kill(); // kill the process.
        });

        /* add process to children for tracking */
        xipframe.children.push(child);

        return;
    };


};
