﻿var util = require('util');

/**
* Displays the Xipframe diagnostic information.
* 
* @module diagnostics
* @param {object} xipframe - the loaded Xipframe instance.
* @param {boolean} suppressConsole - if true suppresses console output.
* @returns {object}
*/
module.exports = function (xipframe) {

    'use strict';

    var diag = {}
      , _ = xipframe.utils._;

    diag.node = process.version;
    diag.memory = util.inspect(process.memoryUsage());
    diag.uptime = process.uptime();
    diag.versions = process.versions;
    diag.platform = process.platform;
    diag.arch = process.arch;
    diag.pid = process.pid;

    
    console.log('');
    console.log(' ###############################################################');
    console.log(' > Node Version:       ' + diag.node);
    console.log(' > Process Id:         ' + diag.pid);
    console.log(' > Uptime:             ' + diag.uptime);
    console.log(' > Platform:           ' + diag.platform);
    console.log(' > Architecture:       ' + diag.arch);
    console.log(' > Memory Usage:       ' + diag.memory);  
    console.log(' > Module Versions:');
    _.forEach(diag.versions, function (ver, key) {
        console.log('       ' + key + ': ' + ver);
    });   
    console.log(' ###############################################################');
    console.log('');

    return diag;

};