
/**
* Displays the Xipframe package information.
* 
* @module info
* @param {object} xipframe - the loaded Xipframe instance.
* @param {boolean} supporessConsole - if true suppresses output to console.
* @returns {object}
*/
module.exports = function (xipframe, suppressConsole) {

    'use strict';

    var info = xipframe.config.info;
    info.apps = xipframe.config.apps;

    if (!suppressConsole) {
    
        console.log('');
        console.log(' ###############################################################');
        console.log(' > Framework:    ' + info.name);
        console.log(' > Description:  ' + info.description);
        console.log(' > Author:       ' + info.author.name + ' <' + info.author.email + '>');
        console.log(' > Version:      ' + info.version);
        console.log(' > License:      ' + info.license);
        console.log(' > Url:          ' + info.homepage);
        console.log(' > Node Version:  ' + process.version);
        console.log(' ###############################################################');
        console.log('');

        listApps(xipframe.config.apps);
    } 

    function listApps(apps, maxw) {

        // TODO make some util to make more reuseable.

        maxw = maxw || 20;

        console.log(' Installed applications below:');
        console.log(' ---------------------------------------------------------------');
        for (var key in apps) {
            if (apps.hasOwnProperty(key)) {
                var obj = apps[key]
                  , l = (key.length + 8)          
                , spaces = (l < maxw ? maxw - l : 0)
                , name = (spaces > 0 ? key : key.slice(0, key.length - 4) + '... ');
                console.log(' > App: ' + format(name, spaces) + ' Path: ' + obj.paths.app);
            }
        }

        function format(val, spaces) {
            var s = '';
            for (var i = 0; i < spaces; i++) s += ' ';
            return val + s;
        }

    }

    return info;

};
