
/**
* Loads up the user config/module by name, path or package
* @module load
* @param {string} appName - the application name to laod.
* @param {object} xipframe - base config information.
* @param {boolean} force - if true forces upgrade.
*/
module.exports = function (appName, xipframe, force) {

    'use strict';

    var utils = xipframe.utils
        , _ = utils._
        , configuration = require('../configuration')
        , InitError = xipframe.errors.InitError
        , tmpConfig = {}
        , app
        , userConfig
        , env;

    app = xipframe.config.apps[appName] || null;

    /* make sure we found the app */
    if (!app) throw new Error(appName + ' does not appear to be installed exiting...');

    tmpConfig.express = {};
    tmpConfig.appName = appName;
    tmpConfig.io = {};

    /* check version if not there could be changes so wipe all saved config except name and paths. */
    if (!app.version || app.version != xipframe.config.info.version) {
        if (force)
            app = { appName: appName, paths: app.paths };
        else
            throw new InitError('Version mismatch detected. If safe to continue run "xipframe start -f"');
    }
         

    /* merge in the app data */
    _.merge(xipframe.config, app);

    console.log

    /* get user defined configurations */   
    userConfig = configuration.include(xipframe).configure();

    env = userConfig.environment || 'development';

    /* merge in the user defined app config */
    _.merge(tmpConfig, userConfig.app);

    if (userConfig.extend) {
        tmpConfig.express.extend = userConfig.extend.express;
        tmpConfig.io.extend = userConfig.extend.io;
        delete userConfig['extend'];
    }

    /* merge in the remaining modules */
    _.merge(tmpConfig, userConfig);

    /* check for specified environment overloads */
    if (userConfig.environments && userConfig.environments[env]) {
        var envConfig = userConfig.environments[env].app || {};        
        _.merge(envConfig, userConfig.environments[env]);       
        _.merge(tmpConfig, envConfig);
    }

    /* cleanup remove unneeded keys. */
    delete tmpConfig.app;
    delete tmpConfig.environment;

    /* temp fixes eventually these will become forked modules */
    _.forEach(xipframe.config.fixups, function (value, key) {
        if (xipframe.config.fixups[key].fixed === false) {           

            /* make backup copy */
            if (value.backup === true) {             
                try {
                    var target = utils.diskio.readSync(value.target);
                    utils.diskio.writeSync(value.target + '.bak', target);
                } catch (e) {
                    console.log(e);
                }
            }   

            /* run fixup */
            try {
                console.log(' > Fixing: ' + value.target)
                var source = utils.diskio.readSync(value.source);
                utils.diskio.writeSync(value.target, source);
                xipframe.config.fixups[key].fixed = true;
            } catch (e) {
                xipframe.config.fixups[key].fixed = false;
                console.log(e);
            }
        }
    });

    /* merge remaining modules including defaults.  */
    configuration.defaults(xipframe, tmpConfig);

    /* save application config to apps.json */
    xipframe.config.lastLoaded = new Date();
    utils.appdate.save(xipframe.config);


};
