var optimist = require('optimist'),
    path = require('path'),
    util = require('util');

/**
* Parses command lines.
*
* TODO: clean this up not happy with it, needs to be more reusable.
*
* @module parse
* @param {object} xipframe - loaded xipframe instance.
*/
module.exports = function parse(xipframe) {

    var utils = xipframe.utils
     , _ = utils._
     , config = xipframe.config
     , argObj = getArgs(optimist.argv)
     , command = argObj.actions[0] || null
     , InitError = xipframe.errors.InitError
     , InvalidOperationError = xipframe.errors.InvalidOperationError
     , NullObjectError = xipframe.errors.NullObjectErrors;

    /* 
    * if no action push start command 
    * probably started directly using node server.js
    */
    if (argObj.actions.length == 0) {
        command = 'start';
        argObj.actions.push('start');
    }

    /* start */
    if (contains('start, run')) {

        /* if we are starting from another dir */
        var appName = (argObj.actions[1] || config.basename),
            remote = (appName ? true : false),
            force = false;

        /* passing -f on start re-runs fixups */
        if (argObj.options && argObj.options.f) {
            force = true;
            var app = config.apps[appName];
            _.merge(app.fixups, config.fixups)
            _.forEach(app.fixups, function (v, k) {
                v.fixed = false;
            });
        }
 
        /* checks if we are starting the application from foreign directory */
        xipframe.remote = remote;

        /* load app configuration */
        xipframe.load(appName, force);

    }

    /* install */
    if (contains('install, new, create')) {    

        // -c (config), -g (angular), -l(less), -v(specify view), -s(seeds path), -t(templates path);        

        /* if there is no second arg we can't continue */
        if (!argObj.actions[1]) throw new NullObjectError('Application name is required for installation.');

        /* populate the config with defined criteria */
        var appName = argObj.actions[1].toLowerCase()
          , paths = {}
          , loadConfig = false
          , viewEngine = 'ejs'
          , userConfig = {};

        paths.app = config.paths.exec + '/' + appName;
        paths.exec = config.paths.exec;
        paths.root = config.paths.root;
        
        if (argObj.options.s && _.isString(argObj.options.s)) paths.seeds = argObj.options.s;
        if (argObj.options.s && !_.isString(argObj.options.s)) throw new InvalidOperationError('Invalid path argument for "seeds".');

        if (argObj.options.t && _.isString(argObj.options.t)) paths.templates = argObj.options.t;
        if (argObj.options.t && !_.isString(argObj.options.t)) throw new InvalidOperationError('Invalid path argument for "templates".');

        if (argObj.options.v && _.isString(argObj.options.v)) viewEngine = argObj.options.v.replace(/\./i, '').toLowerCase();  
        if (argObj.options.v && !_.isString(argObj.options.v)) throw new InvalidOperationError('Invalid path argument for "view".');

        if (argObj.options.c && _.isString(argObj.options.c)) loadConfig = true;
        if (argObj.options.c && !_.isString(argObj.options.c)) throw new InvalidOperationError('Invalid path argument for "config".');

        userConfig = { 

            appName: appName
           , angular: { enabled: argObj.options.g || false }
           , less: { enabled: argObj.options.l || false }
           , viewEngine: viewEngine
           , session: { key: appName + '.sid' }
           , paths: paths
           , load: loadConfig
           , autoRoute: {
               createViews: true
           }

        };

        /* must use html for AngularJs */
        if (userConfig.angular.enabled === true) {        
            userConfig.viewEngine = 'html';
            userConfig.autoRoute.createViews = false;     
        }


        /* call prototype install function */
        xipframe.install(userConfig);

    }


    /* uninstall */
    if (contains('uninstall, delete')) {

        /* if there is no second arg we can't continue */
        if (!argObj.actions[1]) throw new NullObjectError('Application name is required for removal');

        /* call prototype remove function */
        xipframe.uninstall(argObj.actions[1]);

    }

    /* remove */
    if (contains('remove')) {

        /* if there is no second arg we can't continue */
        if (!argObj.actions[1]) throw new NullObjectError('Application name is required for removal');

        /* call prototype remove function */
        xipframe.uninstall(argObj.actions[1], true);
    }

    /* info */
    if (contains('info, package')) {
        var wRoutes = argObj.options.r || null;
        /* get the Xipframe package information */
        xipframe.info();
    }

    /* generation */
    if (contains('generate, gen')) {

        var genType = argObj.actions[1].toLowerCase() || null
          , genAction = argObj.actions[2].toLowerCase() || null
          , appName = argObj.options.a || config.basename
          , app = config.apps[appName] || null
          , genArguments = {}        
          , rendered = '';
        
        /* must have a type (model, controller, view etc as well as a named arg and application.) */
        if (!genType || !genAction || !app) throw new NullObjectError('Generation, action or application could not be found.');

        /* generate model */
        if (genType == 'model') {

            /* make sure we don't end up with a malformed controller name */
            genAction = genAction.replace(/Model/i, '');
            genAction = genAction.lastIndexOf('.') != -1 ? genAction.slice(0, genAction.indexOf('.')) : genAction;
            genAction = utils.helpers.stringToCase(genAction, 'first');

            var outputPath = app.paths.models + '/' + genAction + 'Model.js'
          , templatePath = app.paths.templates + '/model.ejs';

            /* check if controller already exists */     
            if (utils.diskio.existsSync(outputPath)) throw new InvalidOperationError(genAction + 'Model already exists and cannot be overwritten.');

            /* iterate actions and build properties and attributes */
            genArguments = buildArguments(genAction, argObj.actions, { defType: 'type', defValue: 'String' });

            /* render the templates */
            genArguments.tableName = genAction;
            rendered = utils.helpers.render(templatePath, genArguments);

            /* output the rendered controller */
            utils.diskio.writeSync(outputPath, rendered);
            console.log('');
            console.log(' > ' + genAction + 'Model was successfully created.');
        }

        /* generate controller */
        if (genType == 'controller') {

            /* make sure we don't end up with a malformed controller name */
            genAction = genAction.replace(/Controller/i, '');
            genAction = genAction.lastIndexOf('.') != -1 ? genAction.slice(0, genAction.indexOf('.')) : genAction;
            genAction = utils.helpers.stringToCase(genAction, 'first');

            var outputPath = app.paths.controllers + '/' + genAction + 'Controller.js'
              , templatePath = app.paths.templates + '/controller.ejs';
                         
            /* check if controller already exists */
            if (utils.diskio.existsSync(outputPath)) throw new InvalidOperationError(genAction + ' already exists and cannot be overwritten.');

            /* iterate actions and build properties and attributes */
            genArguments = buildArguments(genAction, argObj.actions, { keyValue: true });

            /* render the templates */
            rendered = utils.helpers.render(templatePath, genArguments);

            /* output the rendered controller */
            utils.diskio.writeSync(outputPath, rendered);
            console.log('');
            console.log(' > ' + genAction + 'Controller was successfully created.');

        }

        /* generate view */
        if (genType == 'view') {
            
            /* valid view generation types "show, showall, destroy, update, create" */
            
            var tmpViewEngine = app.viewEngine === 'html' ? 'ejs' : app.viewEngine /* render html using ejs just change the ext on save */
           , view = genAction.lastIndexOf('.') !== -1 ? genAction.toLowerCase() : genAction.toLowerCase() + '.' + tmpViewEngine
           , view = view.indexOf('/') == 0 ? view : '/' + view
           , viewDir = app.paths.views + view.slice(0, view.lastIndexOf('/'))
           , view = app.paths.views + view
           , overwrite = argObj.options.o
           , viewTemplate = argObj.options.t && argObj.options.t !== true ? '/view_' + argObj.options.t + '.' : '/view.';
           
            
  
            /* make sure the target dir exists */
            if (!utils.diskio.existsSync(viewDir)) utils.diskio.mkdirSync(viewDir);

            if (!overwrite && utils.diskio.existsSync(view)) throw new InvalidOperationError('The specified view already exists. Did you mean to flag "-o" to overwrite?');

            /* get the template */
            
            if (!utils.diskio.existsSync(app.paths.templates + viewTemplate + tmpViewEngine)) throw new NullObjectError('Specified view template could not be found.');
            template = utils.diskio.readSync(app.paths.templates + viewTemplate + tmpViewEngine);

            /* write the template */
            view = app.viewEngine === 'html' ? view.replace('.ejs', '.html') : view;
            utils.diskio.writeSync(view, template);
            console.log('');
            console.log(' > The view "' + view + '" was successfully created.');

        }

    }

    if (contains('diagnostics, diag')) {
        xipframe.diagnostics();
    }

    /* shortcut for downloading files */
    if (contains('download')) {
        var source, target;
        source = argObj.options.s || null;
        target = argObj.options.t || null;

        if (!source || !target) throw new NullObjectError('Invalid or missing source/target.');

        xipframe.download(source, target);
    }


    /* help */
    if (contains('help, usage')) {
        // TODO: build help.
    }    

    //////////////////////////////////////////////////
    /////////////////// HELPERS //////////////////////
    //////////////////////////////////////////////////

    /* save our argv object to Xipframe */
    function getArgs(args) {
        var actions = args._, path = args.$0, options = _.clone(args), actionObj = {};
        delete options['$0'];
        delete options['_'];
        return {
            path: path
          , actions: actions
          , options: options
        };        
    }

    /* parse args and check command */
    function contains(args) {
        var result = false;
        command = command.toLowerCase();
        args = args.toLowerCase().replace(/\s/g, '').split(',');
        if(_.contains(args, command)){
            xipframe.command = xipframe.commands[command];
            result = true;
        }
        return result;
    }

    /* build argument properties */
    function buildArguments(name, args, options) {

        if (!args) return {};

        options = options || {};

        var obj = {}
          , defaults = {
              keyValue: false,
              defType: null,
              defValue: null
          };      

        options = _.extend(defaults, options);       
        obj.name = name;
        obj.properties = [];

        /* filter out the generation type */
        args = _.filter(argObj.actions, function (prop, key) {
            if (key > 2) return prop;
        });

        _.forEach(args, function (arg) {
            var argArr = arg.split(':')
              , property = {};
            property.attributes = [];
            if (options.keyValue) { // builds controller key/values
                property.key = argArr[0]; 
                property.value = argArr[1];
                obj.properties.push(property);
            } else {
                property.key = argArr[0];
                var simpleArg = argArr.length === 1 ? true : false;
                _.forEach(argArr, function (attr, key) { // builds args for models.
                    var attribute = null;          
                    if (key > 0) {                     
                        attribute = {};
                        if (attr.indexOf('=') != -1) {
                            var attrArr = attr.split('=');                         
                            attribute.key = formatArgument(attrArr[0]);
                            attribute.value = formatArgument(attrArr[1]) || null;
                        } else {                  
                            if (!options.defType) {
                                attribute.key = formatArgument(attr);
                                attribute.value = null;
                            } else {
                                attribute.key = options.defType;                     
                                attribute.value = formatArgument(attr);                            
                            }
                        }
                    } else {
                        if (simpleArg) attribute = { key: options.defType, value: options.defValue };                 
                    }
                    if(attribute) property.attributes.push(attribute);
                });
                obj.properties.push(property);
            }              
        });

        return obj;
    }

    /* formats property attributes */
    function formatArgument(arg) {

        if (!arg) return null;
        var isNumber = /^[0-9]+$/i.test(arg)
          , isBoolean = /^(true|false)$/i.test(arg)
          , isType = /^(string|number|boolean|date|array)$/i.test(arg)
          , propName = /^(display|order|index|limit|default)$/i.test(arg)
          , hasQuotes = /(\'|\")/gi.test(arg);

        /* if attribute is a type set to upper case first letter */
        if (isType) 
            arg = utils.helpers.stringToCase(arg, 'first');

        /* probably passing a literal value enclose in quotes */
        if (!isType && !isNumber && !isBoolean && !hasQuotes && !propName) 
            arg = "'" + arg + "'";

        return arg;
    }


};
