﻿
/**
* Uninstall Xipframe application.
* @module uninstall
* @param {object} appName - the application to be uninstall.
* @param {object} xipframe - the loaded Xipframe instance.
*/
module.exports = function (appName, xipframe) {

    'use strict';

    var utils = xipframe.utils,
        apps = xipframe.config.apps,
        app = xipframe.config.apps[appName] || null,
        UninstallError = xipframe.errors.UninstallError;

    if (xipframe.config.platform == 'win32' && appName.toLowerCase() == xipframe.config.basename.toLowerCase()) 
        throw new UninstallError('Command executed from path to be deleted. Please use "cd .." without quotes then re-run command');

    if (!app || !app.paths || !utils.diskio.existsSync(app.paths.app)) 
        throw new UninstallError('Application NOT installed or does NOT exist. If the problem persists verify the app in apps.json in the ' + xipframe.config.paths.root + ' directory.');

    console.log('');
    console.log(' ###############################################################');
    console.log(' > Uninstalling Xipframe application.');
    console.log(' ###############################################################');
    console.log('');

    delete apps[app.appName];
    utils.appdate.save(xipframe.config, true);

    utils.diskio.remove(app.paths.app, function (success) {
        if (success) {
            console.log(' > Application "' + app.appName + '" successfully uninstalled.');
        } else {
            console.log(' > Application "' + app.appName + '" could NOT be uninstalled.');
            /* if failed for some reason write back the app to apps.json */
            apps[app.appName] = app;
            utils.appdate.save(xipframe.config);          
        }
    });    

}; 