
'use strict';

/* TODO: need to add some specific handling for various errors */

var util = require('util'),
    AbstractError,
    DatabaseError,
    DatabaseValidationError,
    InitError,
    InstallError,
    UninstallError,
    NotImplementedError,
    NullObjectError,
    InvalidOperationError;

AbstractError = function (msg, constr, codes) {
    Error.captureStackTrace(this, constr || this);
    this.message = msg || 'An unknown error occured.';
    this.codes = codes || null;
};

util.inherits(AbstractError, Error);
AbstractError.prototype.name = 'Abstract Error';


DatabaseError = function(msg) {
    DatabaseError.super_.call(this, msg, this.constructor);
};

util.inherits(DatabaseError, AbstractError);
DatabaseError.prototype.name = 'Database Error';
DatabaseError.prototype.message = 'Unknown database error';


DatabaseValidationError = function (msg, codes) {
    DatabaseError.super_.call(this, msg, this.constructor, codes);
};

util.inherits(DatabaseValidationError, AbstractError);
DatabaseValidationError.prototype.name = 'Database Validation Error';
DatabaseValidationError.prototype.message = 'Unknown validation error';


InitError = function (msg) {
    InitError.super_.call(this, msg, this.constructor);
};

util.inherits(InitError, AbstractError);
InitError.prototype.name = 'Initialization Error';
InitError.prototype.message = 'Unknown error while initializing Xipframe.';

InstallError = function (msg) {
    InstallError.super_.call(this, msg, this.constructor);
};

util.inherits(InstallError, AbstractError);
InstallError.prototype.name = 'InstallError Error';
InstallError.prototype.message = 'Error encountered while installing application.';

UninstallError = function (msg) {
    UninstallError.super_.call(this, msg, this.constructor);
};

util.inherits(UninstallError, AbstractError);
UninstallError.prototype.name = 'UninstallError Error';
UninstallError.prototype.message = 'Error encountered while uninstalling application.';

NotImplementedError = function (msg) {
    NotImplementedError.super_.call(this, msg, this.constructor);
};

util.inherits(NotImplementedError, AbstractError);
NotImplementedError.prototype.name = 'NotImplementedError Error';
NotImplementedError.prototype.message = 'Method or Function not implemented.';

NullObjectError = function (msg) {
    NullObjectError.super_.call(this, msg, this.constructor);
};

util.inherits(NullObjectError, AbstractError);
NullObjectError.prototype.name = 'NullObjectError Error';
NullObjectError.prototype.message = 'Null object reference.';

InvalidOperationError = function (msg) {
    InvalidOperationError.super_.call(this, msg, this.constructor);
};

util.inherits(InvalidOperationError, AbstractError);
InvalidOperationError.prototype.name = 'InvalidOperationError Error';
InvalidOperationError.prototype.message = 'Operation invalid.';

/**
* Export custom error types.
* @module errors
*/
module.exports = {
    InitError: InitError,
    DatabaseError: DatabaseError,
    DatabaseValidationError: DatabaseValidationError,
    InstallError: InstallError,
    UninstallError: UninstallError,
    NotImplementedError: NotImplementedError,
    NullObjectError: NullObjectError,
    InvalidOperationError: InvalidOperationError
};
