
var includeAll = require('include-all');

/**
* Imports framework module and configurations.
* @module include
* @param {object} xipframe - the loaded Xipframe instance.
* @returns {object}
*/
module.exports = function (xipframe) {

    'use strict';

    var _ = xipframe.utils._,
    config = xipframe.config;

    function includeModules (source, filter, exclude, isPath) {
        if (!source || !filter) return null;
        isPath = isPath || false;
        exclude = exclude || /^(example)$/i;
        var opts = { dirname: source };
        if (!isPath) opts.filter = filter;
        else opts.filterPath = filter;
        if (exclude) opts.exclude = exclude;
        return includeAll(opts);
    }

    return {

        /**
        * Includes user configuration settings.
        * @function configure       
        * @param {regex} filter - regular expression to filter by.
        * @param {regext} exclude - regular expression to exclude by.
        * @augments include
        * @returns {object} 
        */
        configure: function(filter, exclude) {
            filter = filter || /(.+)\.js$/i;
            return includeModules(config.paths.configure, filter, exclude);
        },

        /**
       * Includes user configuration settings.
       * @function middleware       
       * @param {regex} filter - regular expression to filter by.
       * @param {regext} exclude - regular expression to exclude by.
       * @augments include
       * @returns {object} 
       */
        middleware: function (filter, exclude) {
            filter = filter || /(.+)\.js$/i;
            return includeModules(config.paths.middleware, filter, exclude);
        },

        /**
       * Includes user configuration settings.
       * @function controllers       
       * @param {regex} filter - regular expression to filter by.
       * @param {regext} exclude - regular expression to exclude by.
       * @augments include
       * @returns {object} 
       */
        controllers: function (filter, exclude) {
            filter = filter || /(.+Controller)\.js$/i;
            return includeModules(config.paths.controllers, filter, exclude);
        },

        /**
       * Includes user configuration settings.
       * @function models       
       * @param {regex} filter - regular expression to filter by.
       * @param {regext} exclude - regular expression to exclude by.
       * @augments include
       * @returns {object} 
       */
        models: function (filter, exclude) {
            filter = filter || /(.+)\.js$/i;
            var models = includeModules(config.paths.models, filter, exclude);
            delete models['hooks']; // don't need this key.           
            return models;     
        },

        /**
         * Includes user defined handlers.
         * @function handlers       
         * @param {regex} filter - regular expression to filter by.
         * @param {regext} exclude - regular expression to exclude by.
         * @augments include
         * @returns {object} 
         */
        handlers: function (filter, exclude) {
            filter = filter || /(.+)\.js$/i;
            return includeModules(config.paths.handlers, filter, exclude);
        },

        /**
         * Includes user defined routes.
         * @function routes       
         * @param {regex} filter - regular expression to filter by.
         * @param {regext} exclude - regular expression to exclude by.
         * @augments include
         * @returns {object} 
         */
        routes: function (filter, exclude) {
            filter = filter || /(.+)\.js$/i;
            return includeModules(config.paths.routes, filter, exclude);
        },


        /**
         * Includes user defined handlers.
         * @function handlers       
         * @param {regex} filter - regular expression to filter by.
         * @param {regext} exclude - regular expression to exclude by.
         * @augments include
         * @returns {object} 
         */
        security: function (filter, exclude) {
            filter = filter || /(.+)\.js$/i;
            return includeModules(config.paths.security, filter, exclude);
        },

        /**
       * Includes user configuration settings.
       * @function custom       
       * @param {regex} filter - regular expression to filter by.
       * @param {regext} exclude - regular expression to exclude by.
       * @param {boolean} optional - determines if is optional.
       * @param {boolean} isPath - indicates the include is by path.
       * @augments include
       * @returns {object} 
       */
        custom: function (source, filter, exclude, isPath) {
            return includeModules(source, filter, exclude, isPath);
        }      

    };
  

};
