﻿
/**
* Handles storing csrf to locals and cookie if Angular is used.
* @module csrfHandler
* @param {object} xipframe - the loaded xipframe instance.
* @returns {function}
*/
module.exports = function redirectsHandler(xipframe) {

    'use strict';

    var config = xipframe.config
      , utils = xipframe.utils
      , _ = utils._;

    /**
    * Set csrf token to locals and cookie if needed.
    *
    * @function csrfHandlerMiddleware
    * @param {object} req - the http request.
    * @param {object} res - the http response.
    * @param {function} next - continues request.
    * @augments csrfHandler
    */
    return function csrfHandlerMiddleware(req, res, next) {

        if (!config.staticIgnore.regex.test(req.url)) {

            if ('GET' == req.method || 'HEAD' == req.method || 'OPTIONS' == req.method) {

                var _csrf = req.session._csrf;
                res.locals._csrf = _csrf;

                /* Angular needs this set in a cookie */
                if (config.angular.enabled) res.cookie('XSRF-TOKEN', _csrf);
            }

        }

        next();

    };


};