
/**
* File not found handler.
* @module fnfHandler
* @param {object} xipframe - the loaded instance of xipframe.
* @returns {function}
*/
module.exports = function fnfHandler(xipframe) {

    'use strict';

    var config = xipframe.config
      , utils = xipframe.utils
      , fnfConfig = config.handlers.fnf;

    /**
     * Handles files not found for bad paths.
     *
     * @function fnfHandlerMiddleware   
     * @param {object} req - the http request.
     * @param {object} res - the http response.
     * @param {function} next - continues request.
     * @augments fnfHandler
     */
    return function fnfHandlerMiddleware(req, res, next) {

        if (xipframe.config.handlers.fnf.enabled || !xipframe.userHandlers.fnf) {

            if (fnfConfig.redirect.type === 'auto' || req.REST || config.staticIgnore.regex.test(req.url)) {
                if (req.REST) res.json(404, 'Oops requested url "' + req.url + '" could not be found.');
                else res.render('shared/404', { url: req.url, title: res.locals.title || 'File Not Found - 404' });
            } else {
                if (fnfConfig.redirect.type === 'view') {
                    if (fnfConfig.redirect.path) res.render('shared/404', { url: req.url, title: res.locals.title || 'File Not Found - 404' });
                    else throw new Error('Invalid view path for fnf handler.');
                } else if (fnfConfig.redirect.type === 'url') {
                    if (fnfConfig.redirect.path) res.redirect(fnfConfig.redirect.path);
                    else throw new Error('Invalid redirect path for fnf handler.');
                } else {
                    res.render('shared/404', { url: req.url, title: res.locals.title || 'File Not Found - 404' });
                }
            }        
        } else {
            next(); // pass on to user defined file not found handler.
        }                   

    };

};
