
var url = require('url');

/**
* Integrates common locals often used.
* @module localsHandler
* @param {object} xipframe - the loaded xipframe instance.
* @returns {function}
*/
module.exports = function localsHandler(xipframe) {

    'use strict';

    var config = xipframe.config
      , utils = xipframe.utils
      , _ = utils._
      , helpers = utils.helpers;        

    /**
    * Handles various tasks to  prepare request.
    *
    * Tasks such as preparing filters for data queries
    *
    * @function localsHandlerMiddleware
    * @param {object} req - the http request.
    * @param {object} res - the http response.
    * @param {function} next - continues request.
    * @augments localsHandler
    */
    return function localsHandlerMiddleware(req, res, next) {

        /* if using AngularJs we don't want caching on the index */
        if (config.angular.enabled) {
            res.setHeader('Cache-Control', 'no-cache');
            res.setHeader('Pragma', 'no-cache');
            res.setHeader('Expires', 0);
        }

        /* check if requests is REST/json */
        req.REST = req.xhr === true;
        if (req.headers['accept']) req.REST = (req.headers['accept'].indexOf('application/json') !== -1);
        if (req.headers['content-type']) req.REST = (req.headers['content-type'].indexOf('application/json') !== -1);

        /* include the environment in the locals */
        res.locals.system = { environment: config.environment };

        if (!config.staticIgnore.regex.test(req.url)) {

            /* the below are required for the framework */
            var reqUrl = url.parse(req.url)
                , urlSegments
                , title;

            /* check for common orm query params */     
            if (req.query) {

                var filter = {},
                    whereFilter = {},
                    where = {};
                
                _.forEach(req.query, function (value, param) {
                    if (_.contains(['order', 'limit', 'skip', 'page', 'q'], param.toLowerCase())) {
                        param = param.toLowerCase();
                        if (_.contains(['limit', 'skip', 'page'], param)) filter[param] = parseInt(value);
                        else filter[param] = value;
                    } else {
                        where[param] = value;
                    }         
                });

                whereFilter['where'] = where;
                req.filter = _.extend({}, whereFilter, filter);
            }

            if (config.handlers.locals || !xipframe.userHandlers.locals) {

                /* auto generate title */
                var path = reqUrl.pathname.slice(1, reqUrl.pathname.length);
                title = '';
                urlSegments = path == '' ? 'Index'.split('/') : path.split('/');
                _.forEach(urlSegments, function (seg) {
                    title += (helpers.stringToCase(seg, 'first') + ' ');
                });
                res.locals.title = title.slice(0, title.length - 1);

            }
            
            next();

        } else {

            next();

        }

    };

    
  
};
