﻿
var url = require('url');

/**
* Returns redirects handler middleware
* @module redirectsHandler
* @param {object} xipframe - the loaded xipframe instance.
* @returns {function}
*/
module.exports = function redirectsHandler(xipframe) {

    'use strict';

    var config = xipframe.config
      , utils = xipframe.utils
      , _ = utils._
      , redirects = xipframe.express.redirects;

    /**
    * Redirect specified routes.
    *
    * @function redirectsHandlerMiddleware
    * @param {object} req - the http request.
    * @param {object} res - the http response.
    * @param {function} next - continues request.
    * @augments redirectsHandler
    */
    return function redirectsHandlerMiddleware(req, res, next) {

        if (!config.staticIgnore.regex.test(req.url)) {

            var reqUrl = url.parse(req.url);

            if (_.has(redirects, reqUrl.pathname)) {

                res.redirect(redirects[reqUrl.pathname] + reqUrl.search);

            } else {

                next();

            }

        } else {

            next();

        }

    };


};