﻿
/**
* Socket io authorization.
* @module authorization
* @param {object} xipframe - the loaded instance of xipframe.
* @returns {function}
*/
module.exports = function authorization(xipframe) {

    'use strict';

    var config = xipframe.config
      , cookie = require('express/node_modules/cookie')
      , parseSignedCookie = require('express/node_modules/connect').utils.parseSignedCookie
      , ConnectSession = require('express/node_modules/connect').middleware.session.Session;


    /**
    * Socket io authorization handshake
    * @function authorizationHandshake
    * @param {object} handshake - the socket handshake.
    * @param {object} accept
    * @augments authorization
    */
    return function authorizationHandshake(handshake, accept) {

        /* check for handshake */
        if (handshake.headers.cookie) {

            /* parse and store cookie */
            handshake.cookie = cookie.parse(handshake.headers.cookie);
            handshake.sessionID = parseSignedCookie(handshake.cookie[config.session.key], config.session.secret);

            /* get the session */
            xipframe.sessions.store.get(handshake.sessionID, function(err, session) {
                if (err) {
                    throw err;
                } else if (!session) {
                    /* generate a new session */
                    handshake.session = new ConnectSession(handshake, {
                        cookie: { httpOnly: true }
                    });
                    accept(null, true);
                } else {
                    /* create the session/handshake */
                    handshake.session = new ConnectSession(handshake, session);
                    accept(null, true);
                }
            });
        } else {
            throw new Error('Socket attempted to connect without a valid session or connected from a third party domain.');
        }

        accept(null, true);
    };

};