﻿
var Redis = require('socket.io/node_modules/redis')
  , RedisStore = require('socket.io/lib/stores/redis');

/**
* Creates Redis pubsub server
* @module redis
* @param {object} xipframe - the xipframe instance.
*/
module.exports = function redis(xipframe) {

    'use strict';

    var config = xipframe.config,
        _ = xipframe.utils._,
        InitError = xipframe.errors.InitError;
    

    return {
        /**
         * Redis client connection
         * @function connection
         * @param {string} host - the connection host.  
         * @param {string} port - the connection port.
         * @param {object} options - connection options.
         * @param {string|null} password - the authentication password for the connection if specified.
         * @augments redis
         * @returns {object}
         */
        connection: function connection(host, port, options, password) {

            var client, host, port, options;


            /* create the client */
            client = Redis.createClient(port, host, options);

            /* check for password */
            if (password) {
                client.auth(password, function(err) {
                    if (err) throw err;
                });
            }


            /* if we get here and client was called throw error no config */
            if (!client)
                throw new InitError('Invalid or missing Redis configuration for Socket.io');

            return client;

        }

        /**
        * Redis pubsub store creation
        *
        * TODO: allow different configs for pub, sub and client.
        *
        * @function store   
        * @augments redis
        * @returns {object}
        */,
        store: function store() {

            var adapters = config.io.adapters, adapter, pub, sub, client;

            adapter = {
                host: '127.0.0.1',
                port: 6379,
                password: null,
                options: {}
            };

            /* redis is specified but use primary adapter settings */
            if (!adapters.redis && adapters.active === 'redis' && config.adapters.redis)
                _.extend(adapter, config.adapters.redis);
            else if (adapters.active === 'redis' && adapters.redis)
                _.extend(adapter, adapters.redis);
            else
                throw new InitError('Unknown or invalid socket.io adapter configuration.');

            pub = this.connection(adapter.host, adapter.port, adapter.options, adapter.password);
            sub = this.connection(adapter.host, adapter.port, adapter.options, adapter.password);
            client = this.connection(adapter.host, adapter.port, adapter.options, adapter.password);

            if (!pub || !sub || !client) throw new InitError('RedisStore has an invalid or missing configuration.');

            return new RedisStore({ redisPub: pub, redisSub: sub, redisClient: client });

        }
    };

};