﻿
var Socket = require('socket.io');

/**
* Creates a socket.io server.
* @module start
* @param {object} xipframe - the xipframe instance.
*/
module.exports = function start(xipframe) {
    
    'use strict';

    var config = xipframe.config.io
      , log = xipframe.log
      , _ = xipframe.utils._
      , httpServer = xipframe.express.server
      , authorization = require('./authorization')(xipframe)
      , redis = require('./redis')(xipframe)
      , InitError = xipframe.errors.InitError
      , io;

    /* can't attach socket if httpServer is null */
    if (!httpServer) throw new InitError('Cannot attach socket to unknown http server.');

    /* create the server */
    io = Socket.listen(httpServer);

    /* primary configuration */
    io.configure(function () {

        /* check the adapter */
        if (config.adapters.active != 'memory') {

            /* currently only redis supported other than memory */            
            if (config.adapters.active === 'redis') {        
                io.set('store', redis.store());
            } else {
                throw new InitError('Unknown socket.io adapter. Please set adapter to either "redis" or "memory".');
            }

        }

        /* iterate over global environment options */
        _.forEach(config.options, function (value, property) {
            configureOption(property, value);
        });

        /* set the logger */
        io.set('logger', {
            error: log.error
          , warn: log.warn
          , info: log.info
          , debug: log.verbose
        });

    });


    /* helper function to build up configuration/options */
    function configureOption(property, value) {

        if (property !== 'authorization') {

            /* just enable boolean settings */
            if (_.isBoolean(value)) {
                if (value)
                    io.enable(property);
                else
                    io.disable(property);
            } else {

                /* set the property value */
                io.set(property, value);
            }

        } else {

            if (value) {
                /* set authorization if enabled */
                io.set('authorization', authorization);
            }

        }

    }

    /* check for user defined extensions */
    if (config.extend) config.extend(io);

    /* set the socket.io server instance */
    xipframe.io.server = io;


};

