var winston = require('winston')
  , _ = require('lodash')
  , errors = require('../configuration').errors
  , fs = require('fs');

/**
* Logger singleton instance class.
* @see {@link module:logger|logger module} for public exposed methods.
* @class Logger
*/
var Logger = (function () {

    'use strict';

    function Logger(options) {

        var level = options.level || 'info'
          , path = options.path || 'server.log'
          , maxsize = options.maxsize || 1024 * 1024 * 10
          , handleExceptions = options.handleExceptions || false
          , NotImplementedError = errors.NotImplementedError
          , transports = []
          , console
          , file;

        if (_.contains(options.transports, 'console')) {
            console = new (winston.transports.Console)({ level: level, colorize: 'true', handleExceptions: handleExceptions, prettyPrint: true });
            transports.push(console);
        }

        if (_.contains(options.transports, 'file')) {
            file = new (winston.transports.File)({ level: level, filename: path, maxsize: maxsize, handleExceptions: handleExceptions, timestamp: true, prettyPrint: true });
            transports.push(file);
        }

        if (_.contains(options.transports, 'mongodb')) {
            throw new NotImplementedError('Mongodb transport for logger not implemented in this version of Xipframe.');
        }

        if (_.contains(options.transports, 'mail')) {
            throw new NotImplementedError('Mail transport for logger not implemented in this version of Xipframe.');
        }

        if (_.contains(options.transports, 'riak')) {
            throw new NotImplementedError('Riak transport for logger not implemented in this version of Xipframe.');
        }

        return new (winston.Logger)({
            transports: transports
        });
        
    };
    
    return {
        /**
        * Gets the Logger.
        * @function
        * @param {object} options - logger options.
        * @augments Logger
        * @returns {object}
        */
        get: function (options) {     

            var logger, console, file
            logger = new Logger(options);

            options.transports = ['console'];
            console = new Logger(options);
            options.transports = ['file'];
            file = new Logger(options);

            function reader(callback) {
                if (!callback) return [];
                var log, path, transport;         
                transport = logger.file.transports.file;         
                path = transport.dirname + '/' + transport.filename;
                fs.readFile(path, 'utf8', function (err, data) {
                    if (err) {
                        throw err;
                    } else {
                        data = data.toString().split('\n') || {};
                        var logs = [];
                        for (var i = 0; i < data.length - 1; i++) {
                            var item = JSON.parse(data[i]);
                            item.id = i;
                            logs.push(item);
                        }
                        callback(logs);
                    }
                });
            }

            logger.reader = reader
            logger.console = console;
            logger.file = file;

            return logger;

        }
    };



})();

/**
* Logging module using winston
* @module logger
*/
module.exports = Logger;
