
/**
* @file example model configuration.
*/
var UserModel = {
    
    // supported datatypes: String, Number, Boolean, Date, Array
    // NOTE: jugglingdb types "Schema.Text", "Text" or "JSON" CANNOT be used
    // in auto generated routes/models at this time. Please custom model/controller/route if
    // you need these features.
    //
    // property attributes: 
    //      type: the data type.
    //      default: a default value.
    //      index: indicates the property is indexed.
    //      length: limits length
    //      order: sets the order in which the property is displayed.
    //      display: sets how labels will display this property name.

    tableName: '',                  // optional name can be specified. if changed it is the name the model will be referenced by.

    description: 'No description.',               // placeholder for description about module. used in auto routing.

    properties: {                   // define each property for model below.

      username: {
          type: String,
          index: true               // indexes this property.
      },

      firstName: {
          type: String,
          length: 15,
          order: 1              // model will be reordered so that first name is the first property.
      },

      lastName: {
          type: String,
          length: 25,
          display: 'Last Name'  // auto generated views will create property labels with this value. 
                                // You can also set "headerToTitle: true" in your configuration under "autoRoutes"
                                // This will cause "lastName" to become "Last Name". This only changes the display name
                                // your database will still see the property as "lastName"
      }

    },

    callbacks: {

        /* 
        * lifecycle callbacks that can be fired before and after ORM events.
        *
        *   - afterInitialize
        *   - beforeCreate
        *   - afterCreate
        *   - beforeSave
        *   - afterSave
        *   - beforeUpdate
        *   - afterUpdate
        *   - beforeDestroy
        *   - afterDestroy
        *   - beforeValidation
        *   - afterValidation
        */

        beforeSave: function (next, Model) {
            /* do something with the model then next to continue */
            next();
        }

    },


    /* 
    * use this to extend your model with validations and relationships.
    * you can also create custom prototype properties.
    * the schema is also accesible which will provide access to 
    * all properties and objects in your ORM.
    */
    extend: function (User, Schema) {

        User.validatesPresenceOf('username', 'firstname', 'lastname');

    },
    
    /*
    ****************************************************
    ***************  DANGER WILL ROBINSON **************
    ****************************************************
    *
    * Enabling seeds destroys all data in table prior 
    * to seeding in the below
    */
    seeds: {
        enabled: false, 
        data: [
            { firstName: "Bob", lastName: "Jones", email: "bjones@aol.com" },
            { firstName: "Larry", lastName: "Carter", email: "carterl@gmail.com" }
        ]
    }

    
};

module.exports = UserModel;        
