/**
* @file global lifecycle callbacks to be added to all models.
*
* This module is good for things like updating modified dates
* on model update or other common lifecycle events such as validation.
*
* lifecycle callbacks:
*   - afterInitialize
*   - beforeCreate
*   - afterCreate
*   - beforeSave
*   - afterSave
*   - beforeUpdate
*   - afterUpdate
*   - beforeDestroy
*   - afterDestroy
*   - beforeValidation
*   - afterValidation
*
*/
module.exports = {

    beforeCreate: function (next, model) {
        this.created = model.created = Date.now();
        this.modified = model.modified = Date.now();
        next();
    },

    beforeUpdate: function (next, model) {
        this.modified = model.modified = Date.now();
        next();
    }

};
